@@ -3,6 +3,8 @@
 //@line 41 "/export/home/mozilla/packages/BUILD/SFWthunderbird/comm-release/mozilla/toolkit/mozapps/extensions/nsBlocklistService.js"
 */
 
+"use strict";
+
 const Cc = Components.classes;
 const Ci = Components.interfaces;
 const Cr = Components.results;
@@ -18,6 +20,7 @@ const KEY_APPDIR                      = "XCurProcD";
 const FILE_BLOCKLIST                  = "blocklist.xml";
 const PREF_BLOCKLIST_LASTUPDATETIME   = "app.update.lastUpdateTime.blocklist-background-update-timer";
 const PREF_BLOCKLIST_URL              = "extensions.blocklist.url";
+const PREF_BLOCKLIST_ITEM_URL         = "extensions.blocklist.itemURL";
 const PREF_BLOCKLIST_ENABLED          = "extensions.blocklist.enabled";
 const PREF_BLOCKLIST_INTERVAL         = "extensions.blocklist.interval";
 const PREF_BLOCKLIST_LEVEL            = "extensions.blocklist.level";
@@ -69,7 +72,7 @@ XPCOMUtils.defineLazyGetter(this, "gABI", function bls_gABI() {
   catch (e) {
     LOG("BlockList Global gABI: XPCOM ABI unknown.");
   }
-//@line 118 "/export/home/mozilla/packages/BUILD/SFWthunderbird/comm-release/mozilla/toolkit/mozapps/extensions/nsBlocklistService.js"
+//@line 121 "/export/home/mozilla/packages/BUILD/SFWthunderbird/comm-release/mozilla/toolkit/mozapps/extensions/nsBlocklistService.js"
   return abi;
 });
 
@@ -372,6 +375,28 @@ Blocklist.prototype = {
     return Ci.nsIBlocklistService.STATE_NOT_BLOCKED;
   },
 
+  /* See nsIBlocklistService */
+  getAddonBlocklistURL: function(id, version, appVersion, toolkitVersion) {
+    if (!gBlocklistEnabled)
+      return "";
+
+    if (!this._addonEntries)
+      this._loadBlocklist();
+
+    let blItem = this._addonEntries[id];
+    if (!blItem || !blItem.blockID)
+      return null;
+
+    return this._createBlocklistURL(blItem.blockID);
+  },
+
+  _createBlocklistURL: function(id) {
+    let url = Services.urlFormatter.formatURLPref(PREF_BLOCKLIST_ITEM_URL);
+    url = url.replace(/%blockID%/g, id);
+
+    return url;
+  },
+
   notify: function(aTimer) {
     if (!gBlocklistEnabled)
       return;
@@ -561,7 +586,7 @@ Blocklist.prototype = {
   },
 
   /**
-//@line 658 "/export/home/mozilla/packages/BUILD/SFWthunderbird/comm-release/mozilla/toolkit/mozapps/extensions/nsBlocklistService.js"
+//@line 683 "/export/home/mozilla/packages/BUILD/SFWthunderbird/comm-release/mozilla/toolkit/mozapps/extensions/nsBlocklistService.js"
    */
 
   _loadBlocklistFromFile: function(file) {
@@ -650,6 +675,8 @@ Blocklist.prototype = {
     // extension for the current application.
     if (result[id].length == 0)
       result[id].push(new BlocklistItemData(null));
+
+    result[id].blockID = blocklistElement.getAttribute("blockID");
   },
 
   _handlePluginItemNode: function(blocklistElement, result) {
@@ -659,7 +686,8 @@ Blocklist.prototype = {
     var matchNodes = blocklistElement.childNodes;
     var blockEntry = {
       matches: {},
-      versions: []
+      versions: [],
+      blockID: null,
     };
     var hasMatch = false;
     for (var x = 0; x < matchNodes.length; ++x) {
@@ -685,6 +713,9 @@ Blocklist.prototype = {
     // Add a default versionRange if there wasn't one specified
     if (blockEntry.versions.length == 0)
       blockEntry.versions.push(new BlocklistItemData(null));
+
+    blockEntry.blockID = blocklistElement.getAttribute("blockID");
+
     result.push(blockEntry);
   },
 
@@ -751,6 +782,34 @@ Blocklist.prototype = {
     return Ci.nsIBlocklistService.STATE_NOT_BLOCKED;
   },
 
+  /* See nsIBlocklistService */
+  getPluginBlocklistURL: function(plugin) {
+    if (!gBlocklistEnabled)
+      return "";
+
+    if (!this._pluginEntries)
+      this._loadBlocklist();
+
+    for each (let blockEntry in this._pluginEntries) {
+      let matchFailed = false;
+      for (let name in blockEntry.matches) {
+        if (!(name in plugin) ||
+            typeof(plugin[name]) != "string" ||
+            !blockEntry.matches[name].test(plugin[name])) {
+          matchFailed = true;
+          break;
+        }
+      }
+
+      if (!matchFailed) {
+        if(!blockEntry.blockID)
+          return null;
+        else
+          return this._createBlocklistURL(blockEntry.blockID);
+      }
+    }
+  },
+
   _blocklistUpdated: function(oldAddonEntries, oldPluginEntries) {
     var addonList = [];
 
@@ -798,7 +857,8 @@ Blocklist.prototype = {
           icon: addons[i].iconURL,
           disable: false,
           blocked: state == Ci.nsIBlocklistService.STATE_BLOCKED,
-          item: addons[i]
+          item: addons[i],
+          url: self.getAddonBlocklistURL(addons[i].id),
         });
       }
 
@@ -834,7 +894,8 @@ Blocklist.prototype = {
               icon: "chrome://mozapps/skin/plugins/pluginGeneric.png",
               disable: false,
               blocked: state == Ci.nsIBlocklistService.STATE_BLOCKED,
-              item: plugins[i]
+              item: plugins[i],
+              url: self.getPluginBlocklistURL(plugins[i]),
             });
           }
         }
@@ -865,25 +926,40 @@ Blocklist.prototype = {
       // This lets the dialog get the raw js object
       args.wrappedJSObject = args;
 
-      var ww = Cc["@mozilla.org/embedcomp/window-watcher;1"].
-               getService(Ci.nsIWindowWatcher);
-      ww.openWindow(null, URI_BLOCKLIST_DIALOG, "",
-                    "chrome,centerscreen,dialog,modal,titlebar", args);
+      /*
+        Some tests run without UI, so the async code listens to a message
+        that can be sent programatically
+      */
+      let applyBlocklistChanges = function() {
+        for (let i = 0; i < addonList.length; i++) {
+          if (!addonList[i].disable)
+            continue;
 
-      for (let i = 0; i < addonList.length; i++) {
-        if (!addonList[i].disable)
-          continue;
+          if (addonList[i].item instanceof Ci.nsIPluginTag)
+            addonList[i].item.disabled = true;
+          else
+            addonList[i].item.softDisabled = true;
+        }
 
-        if (addonList[i].item instanceof Ci.nsIPluginTag)
-          addonList[i].item.disabled = true;
-        else
-          addonList[i].item.softDisabled = true;
+        if (args.restart)
+          restartApp();
+
+        Services.obs.notifyObservers(self, "blocklist-updated", "");
+        Services.obs.removeObserver(applyBlocklistChanges, "addon-blocklist-closed");
       }
 
-      if (args.restart)
-        restartApp();
+      Services.obs.addObserver(applyBlocklistChanges, "addon-blocklist-closed", false)
 
-      Services.obs.notifyObservers(self, "blocklist-updated", "");
+      function blocklistUnloadHandler(event) {
+        if (event.target.location == URI_BLOCKLIST_DIALOG) {
+          applyBlocklistChanges();
+          blocklistWindow.removeEventListener("unload", blocklistUnloadHandler);
+        }
+      }
+
+      let blocklistWindow = Services.ww.openWindow(null, URI_BLOCKLIST_DIALOG, "",
+                              "chrome,centerscreen,dialog,titlebar", args);
+      blocklistWindow.addEventListener("unload", blocklistUnloadHandler, false);
     });
   },
 
